import _classPrivateMethodInitSpec from "@babel/runtime-corejs3/helpers/classPrivateMethodInitSpec";
import _classPrivateMethodGet from "@babel/runtime-corejs3/helpers/classPrivateMethodGet";
import _objectSpread from "@babel/runtime-corejs3/helpers/objectSpread2";
import _classPrivateFieldInitSpec from "@babel/runtime-corejs3/helpers/classPrivateFieldInitSpec";
import _classPrivateFieldGet from "@babel/runtime-corejs3/helpers/classPrivateFieldGet";
import _classPrivateFieldSet from "@babel/runtime-corejs3/helpers/classPrivateFieldSet";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _JSON$stringify from "@babel/runtime-corejs3/core-js-stable/json/stringify";
import _WeakMap from "@babel/runtime-corejs3/core-js-stable/weak-map";
import _WeakSet from "@babel/runtime-corejs3/core-js-stable/weak-set";
var _validators = new _WeakMap();
class ValidatorHandler {
  constructor(validators) {
    _classPrivateFieldInitSpec(this, _validators, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _validators, validators);
  }
  validate(data) {
    var validators = _classPrivateFieldGet(this, _validators);
    for (var validator of validators) {
      validator.validate(data);
    }
  }
}
class InvalidTypeError extends Error {
  constructor(message) {
    super(message);
    this.name = 'Invalid Type';
  }
}
class DoesNotContainPropertyError extends Error {
  constructor(message) {
    super(message);
    this.name = 'Does not contain property';
  }
}
var _requiredPropertyName = new _WeakMap();
class ContainsPropertyValidator {
  constructor(requiredPropertyName) {
    _classPrivateFieldInitSpec(this, _requiredPropertyName, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _requiredPropertyName, requiredPropertyName);
  }
  getRequiredPropertyName() {
    return _classPrivateFieldGet(this, _requiredPropertyName);
  }
  validate(data) {
    var requiredPropertyName = _classPrivateFieldGet(this, _requiredPropertyName);
    if (!(requiredPropertyName in data)) {
      var _context;
      var message = _concatInstanceProperty(_context = "".concat(data, " should include the ")).call(_context, requiredPropertyName, " property");
      throw new DoesNotContainPropertyError(message);
    }
  }
}
var _validType = new _WeakMap();
class IsTypeValidator {
  constructor(validType) {
    _classPrivateFieldInitSpec(this, _validType, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _validType, validType);
  }
  validate(data) {
    var validType = _classPrivateFieldGet(this, _validType);
    if (typeof data !== validType) {
      var _context2;
      var message = _concatInstanceProperty(_context2 = "".concat(data, " should be ")).call(_context2, validType);
      throw new InvalidTypeError(message);
    }
  }
}
var _containsPropertyValidator = new _WeakMap();
var _isTypeValidator = new _WeakMap();
class PropertyContainsValidTypeValidator {
  constructor(containsPropertyValidator, isTypeValidator) {
    _classPrivateFieldInitSpec(this, _containsPropertyValidator, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _isTypeValidator, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _containsPropertyValidator, containsPropertyValidator);
    _classPrivateFieldSet(this, _isTypeValidator, isTypeValidator);
  }
  validate(data) {
    var containsPropertyValidator = _classPrivateFieldGet(this, _containsPropertyValidator);
    containsPropertyValidator.validate(data);
    var requiredPropertyName = containsPropertyValidator.getRequiredPropertyName();
    var requiredValue = data[requiredPropertyName];
    _classPrivateFieldGet(this, _isTypeValidator).validate(requiredValue);
  }
}
var _serializers = new _WeakMap();
class ReplacerBuilder {
  constructor() {
    _classPrivateFieldInitSpec(this, _serializers, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _serializers, {});
  }
  setSerializers(serializers) {
    _classPrivateFieldSet(this, _serializers, serializers);
  }
  build() {
    var serializers = _classPrivateFieldGet(this, _serializers);
    var getSerializerType = unserializedData => {
      var type = typeof unserializedData;
      return type === 'object' ? unserializedData.constructor.name : type;
    };
    return function replacer(key, value) {
      var unserializedData = this[key];
      var serializerType = getSerializerType(unserializedData);
      var serializer = serializers[serializerType];
      if (serializer) {
        var serializedData = serializer.serialize(unserializedData);
        return _objectSpread({
          __typeof__: serializerType
        }, serializedData);
      }
      return value;
    };
  }
}
var _serializers2 = new _WeakMap();
class ReviverBuilder {
  constructor() {
    _classPrivateFieldInitSpec(this, _serializers2, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _serializers2, {});
  }
  setSerializers(serializers) {
    _classPrivateFieldSet(this, _serializers2, serializers);
  }
  build() {
    var serializers = _classPrivateFieldGet(this, _serializers2);
    return function reviver(key, value) {
      var serializerType = value.__typeof__;
      var serializer = serializers[serializerType];
      if (serializer) {
        return serializer.parse(value);
      }
      return value;
    };
  }
}
var _serializers3 = new _WeakMap();
var _serializerValidatorHandler = new _WeakMap();
var _validate = new _WeakSet();
class SerializerHandler {
  constructor(_serializerValidatorHandler2) {
    _classPrivateMethodInitSpec(this, _validate);
    _classPrivateFieldInitSpec(this, _serializers3, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _serializerValidatorHandler, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _serializers3, {});
    _classPrivateFieldSet(this, _serializerValidatorHandler, _serializerValidatorHandler2);
  }
  getSerializers() {
    var serializers = _classPrivateFieldGet(this, _serializers3);
    return _objectSpread({}, serializers);
  }
  addSerializer(serializer) {
    _classPrivateMethodGet(this, _validate, _validate2).call(this, serializer);
    var serializerType = serializer.getSerializerType();
    _classPrivateFieldGet(this, _serializers3)[serializerType] = serializer;
  }
}
function _validate2(serializer) {
  var serializerValidatorHandler = _classPrivateFieldGet(this, _serializerValidatorHandler);
  if (serializerValidatorHandler) {
    serializerValidatorHandler.validate(serializer);
  }
}
var _replacerBuilder = new _WeakMap();
var _reviverBuilder = new _WeakMap();
var _serializerHandler = new _WeakMap();
var _replacer = new _WeakMap();
var _reviver = new _WeakMap();
var _refreshJsonSerializer = new _WeakSet();
var _refreshReplacer = new _WeakSet();
var _refreshReviver = new _WeakSet();
var _installSerializers = new _WeakSet();
var _addSerializer = new _WeakSet();
class JsonSerializer {
  constructor(_replacerBuilder2, _reviverBuilder2, _serializerHandler2) {
    _classPrivateMethodInitSpec(this, _addSerializer);
    _classPrivateMethodInitSpec(this, _installSerializers);
    _classPrivateMethodInitSpec(this, _refreshReviver);
    _classPrivateMethodInitSpec(this, _refreshReplacer);
    _classPrivateMethodInitSpec(this, _refreshJsonSerializer);
    _classPrivateFieldInitSpec(this, _replacerBuilder, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _reviverBuilder, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _serializerHandler, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _replacer, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldInitSpec(this, _reviver, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldSet(this, _replacerBuilder, _replacerBuilder2);
    _classPrivateFieldSet(this, _reviverBuilder, _reviverBuilder2);
    _classPrivateFieldSet(this, _serializerHandler, _serializerHandler2);
  }
  getSerializers() {
    var serializerHandler = _classPrivateFieldGet(this, _serializerHandler);
    return serializerHandler.getSerializers();
  }
  installSerializersAndRefreshJsonSerializer(serializersInstaller) {
    var installOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    _classPrivateMethodGet(this, _installSerializers, _installSerializers2).call(this, serializersInstaller, installOptions);
    _classPrivateMethodGet(this, _refreshJsonSerializer, _refreshJsonSerializer2).call(this);
  }
  addSerializerAndRefreshJsonSerializer(serializer) {
    _classPrivateMethodGet(this, _addSerializer, _addSerializer2).call(this, serializer);
    _classPrivateMethodGet(this, _refreshJsonSerializer, _refreshJsonSerializer2).call(this);
  }
  serialize(unserializedData, space) {
    var replacer = _classPrivateFieldGet(this, _replacer);
    return _JSON$stringify(unserializedData, replacer, space);
  }
  parse(serializedData) {
    var reviver = _classPrivateFieldGet(this, _reviver);
    return JSON.parse(serializedData, reviver);
  }
}
function _refreshJsonSerializer2() {
  var serializerHandler = _classPrivateFieldGet(this, _serializerHandler);
  var serializers = serializerHandler.getSerializers();
  _classPrivateMethodGet(this, _refreshReplacer, _refreshReplacer2).call(this, serializers);
  _classPrivateMethodGet(this, _refreshReviver, _refreshReviver2).call(this, serializers);
}
function _refreshReplacer2(serializers) {
  var replacerBuilder = _classPrivateFieldGet(this, _replacerBuilder);
  replacerBuilder.setSerializers(serializers);
  _classPrivateFieldSet(this, _replacer, replacerBuilder.build());
}
function _refreshReviver2(serializers) {
  var reviverBuilder = _classPrivateFieldGet(this, _reviverBuilder);
  reviverBuilder.setSerializers(serializers);
  _classPrivateFieldSet(this, _reviver, reviverBuilder.build());
}
function _installSerializers2(serializersInstaller, installOptions) {
  var serializerHandler = _classPrivateFieldGet(this, _serializerHandler);
  serializersInstaller.install(serializerHandler, installOptions);
}
function _addSerializer2(serializer) {
  var serializerHandler = _classPrivateFieldGet(this, _serializerHandler);
  serializerHandler.addSerializer(serializer);
}
class JsonSerializerFactory {
  static createJsonSerializer() {
    var serializerValidators = [new PropertyContainsValidTypeValidator(new ContainsPropertyValidator('getSerializerType'), new IsTypeValidator('function')), new PropertyContainsValidTypeValidator(new ContainsPropertyValidator('serialize'), new IsTypeValidator('function')), new PropertyContainsValidTypeValidator(new ContainsPropertyValidator('parse'), new IsTypeValidator('function'))];
    return new JsonSerializer(new ReplacerBuilder(), new ReviverBuilder(), new SerializerHandler(new ValidatorHandler(serializerValidators)));
  }
}
export { JsonSerializerFactory };
//# sourceMappingURL=index.mjs.map
